import * as React from 'react'
import * as PropTypes from 'prop-types'
import A2HSContext from './A2HSContext'

export interface A2HSState {
  deferredPrompt: Event
  isAppInstallable: boolean
  isAppInstalled: boolean
}

const DoNothing: any = () => { }

const initialState: A2HSState = {
  deferredPrompt: DoNothing,
  isAppInstallable: false,
  isAppInstalled: false
}

const A2HSProvider = ({ children }) => {
  const [state, setState] = React.useState(initialState)

  window.addEventListener('beforeinstallprompt', e => {
    // Prevent Chrome 67 and earlier from automatically showing the prompt
    e.preventDefault()
    // Stash the event so it can be triggered later.
    setState({ ...state, deferredPrompt: e, isAppInstallable: true })
    console.log('deffered prompt saved')
  })

  window.addEventListener('appinstalled', evt => {
    setState({ ...state, isAppInstalled: true })
  })

  return <A2HSContext.Provider value={{ ...state, setState }}>{children}</A2HSContext.Provider>
}

export default A2HSProvider