export const INSTRUCTION_PROPERTY = 'dragNDrop';

// TODO: have other typical copy properties?
export const VALUE_PROPERTY = 'value';

interface Instruction {
  [key: string]: string
}

export const createDragSource = (instructionParam: Instruction, copyProperties: any) => ({
  beginDrag(source) {
    const item = {};
    (copyProperties || []).forEach(property => {
      item[property] = source[property];
    });
    const instruction: Instruction = instructionParam || source[INSTRUCTION_PROPERTY];
    if (instruction) {
      item[INSTRUCTION_PROPERTY] = {};
      Object.entries(instruction).forEach(([key, qProperty]) => {
        item[INSTRUCTION_PROPERTY][key] = qProperty
          .split('.')
          .reduce((value, accessor) => value[accessor], source);
      });
    }
    return item;
  }
});

export const createDropTarget = (canDropHandler, dropHandler, copyProperties) => {
  const createArguments = (source, monitor) => {
    const result = [source];
    const item = monitor.getItem();
    if (copyProperties) {
      result.push(copyProperties.map(property => item[property]));
    }
    const instruction = item[INSTRUCTION_PROPERTY];
    if (instruction) {
      result.push(...Object.values(instruction));
    }
    return result;
  };
  return {
    canDrop(source, monitor) {
      return canDropHandler(...createArguments(source, monitor));
    },
    drop(source, monitor) {
      return dropHandler(...createArguments(source, monitor));
    }
  };
};

export function collectTarget(connect, monitor) {
  return {
    connectDropTarget: connect.dropTarget(),
    canDrop: monitor.canDrop()
  };
}

export function collectSource(connect, monitor) {
  return {
    connectDragSource: connect.dragSource(),
    isDragging: monitor.isDragging()
  };
}
