import * as React from 'react';
import classNames from 'classnames';
import { DragSource } from 'react-dnd';
import { VALUE_PROPERTY, collectSource, createDragSource } from '../DnD/helpers';

const getCellId = (column, rowIndex) => `cell-${column.colId}-${rowIndex}`;

export const createCellRenderer = (WrappedComponent) => (props) => {
  const { column, columnApi, rowIndex, onContextMenu, children, ...otherProps } = props;
  const allColumns = columnApi.getAllDisplayedColumns();
  const index = allColumns[0].colDef.type === 'actionsColumn' ? 1 : 0;
  const canExpand = props.node.canFlower && column.colId === allColumns[index].colId;

  return (
    <div id={getCellId(column, rowIndex)}
         onContextMenu={onContextMenu}>
      <WrappedComponent canExpand={canExpand} {...otherProps}/>
      {children}
    </div>
  );
};

class DefaultCellRenderer extends React.Component<any, any> {
  constructor(props) {
    super(props);
    this.state = {
      expanded: false
    };
  }

  render() {
    const { node, canExpand, value, format, connectDragSource } = this.props;

    const onExpand = () => {
      const expanded = !node.expanded;
      node.setExpanded(expanded);
      this.setState({
        expanded
      });
    };

    const arrow = (
      <span onClick={onExpand}
            className={classNames('arrow', {
              'arrow-right': !this.state.expanded,
              'arrow-down': this.state.expanded
            })}/>
    );

    let newValue = value;
    let cssClass = 'cell-text';
    if (value === '') {
      newValue = '(Empty)';
      cssClass = 'cell-text-hidden';
    }
    if (value === null) {
      newValue = '(Null)';
      cssClass = 'cell-text-hidden';
    }

    return connectDragSource(
      <div className='default-cell-container'>
        {canExpand ? arrow : null}
        <span className={cssClass}>
          {format ? format(newValue) : value}
        </span>
      </div>
    );
  }
}

const cellSource = createDragSource(null, [VALUE_PROPERTY, 'colDef']);

// eslint-disable-next-line new-cap
export default DragSource('GRID-CELL', cellSource, collectSource)(DefaultCellRenderer);
