import ColumnModel from './ColumnModel';

// TODO: replace these functions with lodash

function asMap(array, property) {
  const map = {};
  array.forEach(element => {
    const key = typeof element[property] === 'function' ? element[property]() : element[property];
    map[key] = element;
  });
  return map;
}

function objectEquals(x, y) {
  if (x === null || x === undefined || y === null || y === undefined) {
    return x === y;
  }
  // after this just checking type of one would be enough
  if (x.constructor !== y.constructor) {
    return false;
  }
  // if they are functions, they should exactly refer to same one (because of closures)
  if (x instanceof Function) {
    return x === y;
  }
  // if they are regexps, they should exactly refer to same one
  if (x instanceof RegExp) {
    return x === y;
  }
  if (x === y || x.valueOf() === y.valueOf()) {
    return true;
  }
  if (Array.isArray(x) && x.length !== y.length) {
    return false;
  }

  // if they are dates, they must had equal valueOf
  if (x instanceof Date) {
    return false;
  }

  // if they are strictly equal, they both need to be object at least
  if (!(x instanceof Object)) {
    return false;
  }
  if (!(y instanceof Object)) {
    return false;
  }

  // recursive object equality check
  const p = Object.keys(x);
  return Object.keys(y).every(i => p.includes(i)) &&
    p.every(i => objectEquals(x[i], y[i]));
}


function arrayEquals(x, y) {
  if (!Array.isArray(x) && !Array.isArray(y)) {
    return false;
  }
  if (x.length !== y.length) {
    return false;
  }

  return x.every((xObj, i) => objectEquals(xObj, y[i]));
}

export default class ColumnsModel {
  columns: ColumnModel[]
  constructor(columns = []) {
    this.columns = columns;
  }

  getColumns() {
    return this.columns.map(column => new ColumnModel(column));
  }

  setColumns(columns) {
    return new ColumnsModel(columns.map(column => column.serialize()));
  }

  resetColumns(width) {
    const columns = this
      .getColumns()
      .map(column => column.setVisible(true).setPin('').setWidth(width));
    return this.setColumns(columns);
  }

  filter(filterFunction) {
    return this.setColumns(this.getColumns().filter(filterFunction));
  }

  toggleColumnVisibility(idSet) {
    return this.setColumns(
      this.getColumns()
        .map(column => (
          idSet.has(column.getId())
            ? column.setVisible(!column.isVisible())
            : column))
    );
  }

  merge(other) {
    const otherColumns = other.getColumns();
    const curColumns = this.getColumns();
    if (curColumns && curColumns.length > 0) {
      const otherColumnsMap = asMap(otherColumns, 'getId');
      return this.setColumns(curColumns
        .filter((col) => (otherColumnsMap[col.getId()] && delete otherColumnsMap[col.getId()]))
        .concat(Object.values(otherColumnsMap)));
    }
    return this.setColumns(otherColumns);
  }

  isEqual(other) {
    return arrayEquals(this.getColumns(), other.getColumns());
  }

  serialize() {
    return this.getColumns().map(column => column.serialize());
  }
}

export const compareColumns = (column1, column2) => {
  const id1 = column1.getId().toUpperCase();
  const id2 = column2.getId().toUpperCase();
  return id1 < id2 ? -1 : 1;
};
