import SortInfo from './HeaderModel';

const ROW_BUFFER = 1000;
export const LOAD_LIMIT = 500;

export default class GridModel {
  rowData
  headers
  sortInfo
  start
  end
  loading

  constructor({
    rowData,
    headers,
    sortInfo,
    start,
    end,
    loading
  }: any) {
    this.rowData = rowData || null;
    this.headers = headers || [];
    this.sortInfo = sortInfo;
    this.start = start || 0;
    this.end = end || 0;
    this.loading = loading;
  }

  getRowData() {
    return this.rowData;
  }

  getRowDataLength() {
    return this.rowData ? this.rowData.length : 0;
  }

  getHeaders() {
    return this.headers.slice(0);
  }

  setSortInfo(sortInfo) {
    return new GridModel({ ...this, sortInfo: sortInfo.serialize() });
  }

  getSortInfo() {
    return this.sortInfo ? new SortInfo(this.sortInfo) : undefined;
  }

  getStart() {
    return this.start;
  }

  getEnd() {
    return this.end;
  }

  setEmptyData() {
    return new GridModel({ ...this, rowData: null, headers: [] });
  }

  isEmpty() {
    return this.rowData === null;
  }

  setStart(start) {
    return new GridModel({ ...this, start });
  }

  setEnd(end) {
    return new GridModel({ ...this, end });
  }

  setLoading(loading) {
    return new GridModel({ ...this, loading });
  }

  isLoading() {
    return !!this.loading;
  }

  combine(other) {
    const append = this.getStart() <= other.getStart(); // append/prepend
    const partialRowData = append
      ? (this.getRowData() || []).slice(0, other.getStart() - this.getStart())
      : (this.getRowData() || []).slice(other.getEnd() - this.getEnd());
    let rowData = append
      ? partialRowData.concat(other.getRowData())
      : other.getRowData().concat(partialRowData);
    let start = append ? this.getStart() : other.getStart();
    const extraLength = rowData.length - ROW_BUFFER;
    if (extraLength > 0) {
      if (append) {
        start += extraLength;
        rowData = rowData.slice(extraLength);
      } else {
        rowData = rowData.slice(0, ROW_BUFFER);
      }
    }

    return new GridModel({
      ...this.serialize(),
      ...other.serialize(),
      rowData,
      start,
      end: start + rowData.length,
      sortInfo: other.getSortInfo().serialize()
    });
  }

  formatRowDataForGrid(newHeaders) {
    const headers = newHeaders || this.getHeaders();
    const { rowData } = this;
    const newRowData = rowData
      ? rowData.map(row => {
        const convertedRow = {};
        row.forEach((value, index) => {
          convertedRow[headers[index]] = value;
        });
        return convertedRow;
      })
      : rowData;
    return new GridModel({ ...this, headers, rowData: newRowData });
  }

  serialize() {
    return { ...this };
  }
}

export function createGridData(headers, sortInfo, start, end, rowData, loading) {
  return new GridModel({
    rowData,
    headers,
    sortInfo: sortInfo ? sortInfo.serialize() : {},
    start,
    end,
    loading
  });
}
