// aka GridPage
import * as React from 'react';
import { Button } from '@material-ui/core';
import * as PropTypes from 'prop-types';
import MessageBox from '../Popup/MessageBox';
import Grid from './Grid';

const getMessageBoxConfig = (type, callback) => ({
  message: `Are you sure you want to ${type} the selected item(s)?`,
  icon: 'warning',
  buttons: [
    {
      label: type,
      id: `GridPage_MessageBox_${type}_Button`,
      onClick: callback,
      primary: true
    },
    {
      label: 'Cancel',
      id: 'GridPage_MessageBox_Cancel_Button'
    }
  ]
});

const GridPage = (WrappedComponent) =>
  class extends React.Component<any, any> {
    constructor(props) {
      super(props);

      this.state = {
        selectedItems: [],
        messageBoxConfig: {
          hidden: true
        }
      };
      this.getPageButtons = this.getPageButtons.bind(this);
      this.onGridSelection = this.onGridSelection.bind(this);
    }

    onGridSelection(selectedItems) {
      this.setState({
        selectedItems
      });
    }

    getPageButtons() {
      const { buttons = [] } = this.props;
      const { selectedItems } = this.state;

      return buttons.map(({ id, label, showDialog, primary, secondary, onClick, disabled }) => {
        let clickHandler = null; // button disabled
        if (!disabled || (typeof disabled === 'function' && !disabled(selectedItems))) {
          clickHandler = () => {
            const callback = () => {
              onClick(selectedItems);
              this.setState({
                selectedItems: []
              });
            };
            if (showDialog) {
              this.setState({
                messageBoxConfig: {
                  hidden: false,
                  ...getMessageBoxConfig(label.toLowerCase(), callback)
                }
              });
            } else {
              callback();
            }
          };
        }
        return (
          <Button
            key={id}
            id={id}
            color={primary ? 'primary' : secondary ? 'secondary' : undefined}
            onClick={clickHandler}
          >{label}</Button>
        );
      });
    }

    render() {
      const {
        title,
        subtitle,
        children,
        ...otherProps
      } = this.props;
      const { messageBoxConfig } = this.state;

      return (
        <div className='grid-page'>
          <div className='header'>
            <div className='description'>
              <div className='title'>{title}</div>
              <div className='subtitle'>{subtitle}</div>
            </div>
            <div className='buttons'>
              {this.getPageButtons()}
            </div>
          </div>
          <WrappedComponent onRowSelected={this.onGridSelection} {...otherProps} />
          <MessageBox
            onClose={() => this.setState({ messageBoxConfig: { hidden: true } })}
            config={messageBoxConfig}
          />
          {children}
        </div>
      );
    }
  };

GridPage.propTypes = {
  title: PropTypes.string,
  subtitle: PropTypes.string,
  buttons: PropTypes.array
};

/* eslint-disable new-cap */
export default GridPage(Grid);
