import { generateID } from '../../helpers';

export const HEADER_SELECTED = 'header-selected';

export default class HeaderSelection {
  columnApi: any;
  gridApi: any;
  selectedHeaders: Set<any>;
  lastSelectedHeader: any
  constructor(columnApi, gridApi) {
    this.columnApi = columnApi;
    this.gridApi = gridApi;
    this.selectedHeaders = new Set();
    this.lastSelectedHeader = null;
  }

  toggleSelectHeader(column) {
    this.lastSelectedHeader = column;
    if (this.selectedHeaders.has(column)) {
      this.selectedHeaders.delete(column);
    } else {
      this.selectedHeaders.add(column);
    }
    this.applyStyle(column.colId, true);
  }

  selectHeader(column) {
    this.lastSelectedHeader = column;
    this.selectedHeaders.add(column);
    this.applyStyle(column.colId);
  }

  isHeaderSelected(column) {
    return this.selectedHeaders.has(column);
  }

  selectHeadersInRange(toColumn) {
    const allHeaders = this.getAllHeaders();
    const from = allHeaders.indexOf(this.lastSelectedHeader);
    if (from >= 0) {
      const to = allHeaders.indexOf(toColumn);
      const indexes = [from, to].sort((a, b) => a - b);

      for (let i = indexes[0]; i <= indexes[1]; i++) {
        this.selectedHeaders.add(allHeaders[i]);
        this.applyStyle(allHeaders[i].colId);
      }
    }
  }

  getAllHeaders(onlyStickedColumns = false) {
    return onlyStickedColumns
      ? this.columnApi.getDisplayedLeftColumns()
      : this.columnApi.getAllDisplayedColumns();
  }

  getSelectedHeaders(onlyStickedHeaders = false) {
    if (!onlyStickedHeaders) {
      return Array.from(this.selectedHeaders);
    }
    const stickedHeaders = this.getAllHeaders(onlyStickedHeaders);
    const selectedFiltered = [];
    stickedHeaders.forEach((col) => {
      if (this.selectedHeaders.has(col)) {
        selectedFiltered.push(col);
      }
    });
    return selectedFiltered;
  }

  applyStyle(colId: any, toggle?: any) {
    const element = document.querySelector(`#${generateID('Header', colId)}`);
    if (element && toggle) {
      element.classList.toggle(HEADER_SELECTED);
    } else if (element && !element.classList.contains(HEADER_SELECTED)) {
      element.classList.add(HEADER_SELECTED);
    }
  }

  clearSelection() {
    this.selectedHeaders = new Set();
    this.lastSelectedHeader = null;
    for (const element of Array.from(document.getElementsByClassName(HEADER_SELECTED))) {
      element.classList.remove(HEADER_SELECTED);
    }
  }

  pinSelection(shouldPin) {
    const selected = this.getSelectedHeaders();
    this.clearSelection();
    selected.forEach(col => {
      this.columnApi.setColumnPinned(col.colId, shouldPin ? 'left' : null);
    });
    this.redraw();
  }

  hideSelection() {
    this.getSelectedHeaders().forEach(col => {
      this.columnApi.setColumnVisible(col.colId, this.columnApi.getColumn(col.colId).hide);
    });
    this.clearSelection();
    this.redraw();
  }

  redraw() {
    this.gridApi.refreshCells({ force: true });
  }
}
