import * as React from 'react';
import ContextMenu from './ContextMenu';

const withContextMenu = (WrappedComponent, getMenuOptions) =>
  class extends React.Component<any, any> {
    constructor(props) {
      super(props);
      this.onContextMenu = this.onContextMenu.bind(this);
      this.hideContextMenu = this.hideContextMenu.bind(this);
      this.state = {
        menuVisible: false
      };
    }

    onContextMenu(event) {
      event.preventDefault();
      event.stopPropagation();
      this.setState({
        menuVisible: true
      });
    }

    hideContextMenu() {
      this.setState({
        menuVisible: false
      });
    }

    render() {
      return (
        <WrappedComponent onContextMenu={this.onContextMenu} {...this.props}>
          {this.state.menuVisible && <ContextMenu onBlur={this.hideContextMenu}
                                                  options={getMenuOptions(this.props)}/>}
        </WrappedComponent>
      );
    }
  };

export default withContextMenu;
