import * as React from 'react';
import * as PropTypes from 'prop-types';
import classNames from 'classnames';
import Widget from './Widget';
import { generateID } from '../../helpers';
import { WidgetProps } from './useWidget'

const WidgetGrid = (props) => {
  const { dashboard, search, actions, config, placeholder } = props;
  const widgets = dashboard.getFrontItems();
  const displayWidgets = widgets.length === 0 && placeholder
    ? placeholder
    : widgets.map((frontWidget) => {
      const props: WidgetProps = {
        key: generateID('Widget', frontWidget.getId()),
        frontContent: frontWidget,
        backContent: dashboard.getBackItem(frontWidget),
        overlay: dashboard.isOverlay(frontWidget),
        search,
        actions,
        config
      } as any
      return (
        <Widget {...props} />
      )
    })

  const classes = classNames('widget-grid', {
    'full-view': dashboard.isFullView && dashboard.isFullView()
  });

  return (
    <div className={classes}>
      {displayWidgets}
    </div>
  );
};

WidgetGrid.propTypes = {
  dashboard: PropTypes.object.isRequired,
  actions: PropTypes.object.isRequired,
  config: PropTypes.object,
  search: PropTypes.object,
  placeholder: PropTypes.element
};

export default WidgetGrid;
