import { generateUUID } from '../../helpers';
import { Widget } from '../../types'
// import { Chart } from './chart';
// import { createTextBox, TextBox } from './textBox';
// import { Table } from './table';
// import { DatePicker } from './datePicker';
// import { Timeline } from './timeline';
// import { SeSearch } from './seSearch';
// import { Filter } from './filter';

export const WIDGET_TYPE = {
  CHART: 'chart',
  TEXTBOX: 'text',
  TABLE: 'table'
};
/*
const getMetadata = (search) => {
  const { datePicker, timeline, filter, seSearch, queryObject, query } = search;
  return {
    datePicker,
    timeline: new Timeline(timeline).setData().serialize(),
    filter,
    seSearch,
    queryObject,
    query
  };
};
*/
export class WidgetModel implements Widget {
  id: string
  dashboardId: any
  type: string
  name: string
  content: any
  userId: any
  metadata: any
  primary: string
  secondary?: string
  values: Widget['values']

  constructor({
    id,
    dashboardId,
    type,
    name,
    beingRenamed,
    content,
    userId,
    metadata,
    invalid
  }: any = {}) {
    this.id = id;
    this.dashboardId = dashboardId;
    this.type = type;
    this.name = name;
    // this.beingRenamed = beingRenamed;
    this.content = content;
    this.userId = userId;
    this.metadata = metadata;
    // this.invalid = invalid;
  }

  serialize() {
    const {
      id,
      dashboardId,
      type,
      name,
      // beingRenamed,
      content,
      userId,
      metadata,
      // invalid
    } = this;
    return {
      id,
      dashboardId,
      type,
      name,
      // beingRenamed,
      content,
      userId,
      metadata,
      // invalid
    };
  }

  setId(id) {
    return new WidgetModel({
      ...this.serialize(),
      id
    });
  }

  getId() {
    return this.id;
  }

  isOwner(dashboard) {
    return dashboard ? this.dashboardId === dashboard.getId() : false;
  }

  setType(type) {
    return new WidgetModel({
      ...this.serialize(),
      type
    });
  }

  getType() {
    return this.type;
  }

  setName(name) {
    return new WidgetModel({
      ...this.serialize(),
      name
    });
  }

  getName() {
    return this.name;
  }

  /*isBeingRenamed() {
    return !!this.beingRenamed;
  }*/

  setBeingRenamed(beingRenamed) {
    return new WidgetModel({
      ...this.serialize(),
      beingRenamed
    });
  }

  setContent(dataObj) {
    return new WidgetModel({
      ...this.serialize(),
      content: dataObj.serialize()
    });
  }
/*
  getContent() {
    switch (this.getType()) {
      case WIDGET_TYPE.TEXTBOX:
        return new TextBox(this.content);
      case WIDGET_TYPE.CHART:
        return new Chart(this.content);
      case WIDGET_TYPE.TABLE:
        return new Table(this.content);
      default:
        return undefined;
    }
  }
  */
 /*
  getIssues() {
    return this.getContent().getIssues();
  }
  */
/*
  getSearch() {
    const { userId, metadata } = this;
    const name = this.getName();
    const id = this.getId();
    return metadata ? { ...metadata, userId, name, id } : undefined;
  }
*/
  getUserId() {
    return this.userId;
  }
  
  /*
  getDatePicker() {
    const search = this.getSearch();
    return search ? new DatePicker(search.datePicker) : undefined;
  }
  */

  setDatePicker(datePicker) {
    const { metadata } = this;
    return metadata
      ? new WidgetModel({
        ...this,
        metadata: { ...metadata, datePicker: datePicker.serialize() }
      })
      : this;
  }

  /*
  isValid() {
    return !this.invalid;
  }
  */
 /*
  setSearch(search) {
    if (search) {
      return new WidgetModel({
        ...this.serialize(),
        metadata: { ...this.metadata, ...getMetadata(search) }
      });
    }
    return this;
  }
  */

  /*
  getFilter() {
    const search = this.getSearch();
    return search ? new Filter(search.filter) : undefined;
  }
  */

  setFilter(filter) {
    const { metadata } = this;
    return metadata
      ? new WidgetModel({
        ...this,
        metadata: { ...metadata, filter: filter.serialize() }
      })
      : this;
  }

  /*
  hasRefresh() {
    const datePicker = this.getDatePicker();
    return this.isValid() && !!(datePicker && datePicker.isRelative());
  }
  */

  /*
  getSeSearch() {
    if (this.isValid()) {
      const { seSearch } = this.getSearch() || {};
      if (seSearch) {
        return new SeSearch(seSearch);
      }
    }
    return undefined;
  }
  */

  /*
  canRefresh() {
    const seSearch = this.getSeSearch();
    return !!(seSearch && (seSearch.isNew() || !seSearch.isRunningFirstChunk()));
  }
  */

  /*
  refreshDatePicker(endDate) {
    if (this.hasRefresh()) {
      const datePicker = this.getDatePicker().refresh(endDate);
      const metadata = { ...this.metadata, datePicker: datePicker.serialize() };
      return new WidgetModel({ ...this.serialize(), metadata });
    }
    return this;
  }*/
/*
  hasCreateSearch() {
    const search = this.getSearch();
    return !!search;
  }
*/
  /*
  canCreateSearch() {
    const seSearch = this.getSeSearch();
    return !!(seSearch && (seSearch.isDone() || seSearch.isNew()));
  }
  */

  setQuery(query) {
    const { metadata } = this;
    return metadata
      ? new WidgetModel({ ...this, metadata: { ...metadata, query } })
      : this;
  }
/*
  getQuery() {
    const search = this.getSearch() || {};
    return search.query || '';
  }
*/
  setQueryObject(queryObject) {
    const { metadata } = this;
    return metadata
      ? new WidgetModel({ ...this, metadata: { ...metadata, queryObject } })
      : this;
  }
/*
  getQueryObject() {
    const search = this.getSearch() || {};
    return search.queryObject;
  }
*/
  /*
  isRefreshing(includeStopped) {
    const seSearch = this.getSeSearch();
    return seSearch
      && (seSearch.isRunningFirstChunk() || seSearch.isRunningNextChunk())
      && (includeStopped || !seSearch.isRestartable());
  }
  */
}

const createWidget = (dashboard, type, name, content, metadata) => new WidgetModel({
  id: generateUUID(),
  dashboardId: dashboard ? dashboard.getId() : undefined,
  type,
  name,
  content: content.serialize(),
  metadata
});
/*
export function createTextWidget(dashboard, text) {
  return createWidget(dashboard, WIDGET_TYPE.TEXTBOX, 'Text Box', createTextBox(text));
}
*/
/*
function createMetadataWidget(dashboard, type, content, search) {
  const name = content.getName();
  const fields = content.getFields();
  const metadata = getMetadata(search);
  const idSet = new Set();
  fields.forEach(field => idSet.add(field.getId()));
  const filter = new Filter(metadata.filter);
  filter.getFields().forEach(field => idSet.add(field.getId()));
  const fieldIds = Array.from(idSet);
  const newMetadata = { ...metadata, seSearch: { ...metadata.seSearch, fieldIds } };

  return createWidget(dashboard, type, name, content.clone(), newMetadata);
}
*/
/*
export function createTableWidget(dashboard, table, search) {
  return createMetadataWidget(dashboard, WIDGET_TYPE.TABLE, table, search);
}
*/
/*
export function createChartWidget(dashboard, chart, search) {
  return createMetadataWidget(dashboard, WIDGET_TYPE.CHART, chart, search);
}
*/
