import { useState, FunctionComponent, SetStateAction, Dispatch } from 'react'
import { Widget } from '../../types'

export interface WidgetGridActions {
  maximize: (maximized: string) => void
  restore: () => void
}

export interface WidgetGridProps {
  actions: any
  placeholder: any
  overlay: any
  visuals: {
    [key: string]: FunctionComponent
  },
  widgets: {
    [key: string]: Widget
  }
}

export interface WidgetGridState {
  maximized?: string // widget maximized
}

export type WidgetGridEffect = [WidgetGridState | undefined, Dispatch<SetStateAction<WidgetGridState | undefined>>, WidgetGridActions]

export default ({ visuals, widgets }: WidgetGridProps): WidgetGridEffect => {
  
  const [state, setState] =  useState<WidgetGridState>({ })

  const maximize = (maximized: string) => {
    setState({ maximized })
  }

  const restore = () => {
    const { maximized, ...next } = state
    setState(next)
  }

  return [state, setState, {
    maximize,
    restore
  }]
}