import * as React from 'react'
import { ListSelectMixin, ListSelectProps } from './ListSelect'

export interface CheckListSelectProps extends Pick<ListSelectProps, 'dataSet' | 'dataSource' | 'dataSourceKeyToLabel' | 'filter' | 'limit' | 'onChange'> {
}

const Checkbox = ({ key, value, onChange }) => (
  <li key={key}>
    <input checked={value} onChange={(value) => onChange(value, key)} type="checkbox" />
  </li>
)

const Collapse = ({ onClick }) => (
  <li key={'collapse'}>
    <span>Collapse</span>
  </li>
)

const Expand = ({ onClick }) => (
  <li key={'expand'}>
    <span>Expand</span>
  </li>
)

const Filter = ({ value, onChange }) => (
  <li key='filter'>
    <input onChange={onChange} type="text" value={value} />
  </li>
)

const Select = ({ children }) => (
  <ul className="check-list-select">
    {children}
  </ul>
)

const SelectAll = ({ onClick }) => (
  <li key={'select-all'}>
    <span>Collapse</span>
  </li>
)

const UnselectAll = ({ onClick }) => (
  <li key={'unselect-all'}>
    <span>Expand</span>
  </li>
)

export const CheckListSelect = (props: CheckListSelectProps) => {
  const ListSelect = ListSelectMixin(
    {
      CollapseComponent: Collapse,
      ExpandComponent: Expand,
      FilterComponent: Filter,
      OptionComponent: Checkbox,
      SelectComponent: Select,
      SelectAllComponent: SelectAll,
      UnselectAllComponent: UnselectAll,
    }
  )
  return (
    <ul className="check-list-select">
      <ListSelect {...props} />
    </ul>
  )
}