import * as React from 'react'
import { ComponentClass, FunctionComponent } from 'react'
import { ListSelector, ListSelectorOptions } from './ListSelector'

type Clickable = FunctionComponent<{ onChange: () => void }>
type Container = FunctionComponent<{ children: React.ReactElement }>
type Control = FunctionComponent<{ onChange: (value: string) => void }>

export interface ListSelectMixinOptions {
  CollapseComponent: any // Clickable
  ExpandComponent: any // Clickable
  FilterComponent: any // Control
  OptionComponent: any // Control
  SelectComponent: any // Container
  SelectAllComponent: any // Clickable
  UnselectAllComponent: any // Clickable
}

export interface ListSelectProps extends ListSelectorOptions {
  dataSource: object | Array<string>
  limit?: number
  onChange: (selection: Array<string>) => void
  
}

export const ListSelectMixin = ({
  CollapseComponent,
  ExpandComponent,
  FilterComponent,
  OptionComponent,
  SelectComponent,
  SelectAllComponent,
  UnselectAllComponent
}: ListSelectMixinOptions) => ({
  dataSet,
  dataSource,
  dataSourceKeyToLabel,
  limit,
  onChange
}: ListSelectProps) => {
  const [expanded, setExpanded] = React.useState(false)
  const [filter, setFilter] = React.useState()

  const selector = ListSelector(dataSource)
  const selection = selector({
    dataSet,
    dataSourceKeyToLabel,
    filter
  })

  const handleCollapseClick = () => {
    console.log('collapse')
    setExpanded(false)
  }

  const handleExpandClick = () => {
    console.log('expand')
    setExpanded(true)
  }

  const handleFilterChange = (value) => {
    setFilter(value)
  }

  const handleOptionChange = (selected: boolean, key: string) => {
    if (selected) {
      if (!dataSet.has(key)) {
        const prev = Array.from(dataSet)
        const next = [...prev, key]
        console.log('select', key, prev, next)
        onChange(next)
      }
    }
    else {
      if (dataSet.has(key)) {
        const prev = Array.from(dataSet)
        const next = prev.filter(k => k !== key)
        console.log('unselect', key, prev, next)
        onChange(next)
      }
    }
  }

  const handleSelectAllClick = () => {
    const prev = Array.from(dataSet)
    const more = selection.map(({ key }) => key)
    const next = Array.from(new Set([...prev, ...more]))
    console.log('select all', prev.length, more.length, next.length)
    onChange(next)
  }

  const handleUnselectAllClick = () => {
    console.log('unselect all')
    const prev = Array.from(dataSet)
    const less = new Set(selection.map(({ key }) => key))
    const next = prev.filter(less.has)
    console.log('unselect all', prev.length, more.length, next.length)
    onChange(next)
  }

  const more: React.ReactElement[] = []

  if (selection.some(({ key }) => !dataSet.has(key))) {
    more.push(<SelectAllComponent onClick={handleSelectAllClick} />)
  }

  if (selection.some(({ key }) => dataSet.has(key))) {
    more.push(<UnselectAllComponent oncClick={handleUnselectAllClick} />)
  }

  if (Number.isInteger(limit) && selection.length + more.length > limit) {
    if (expanded) {
      more.push(<CollapseComponent onClick={handleCollapseClick} />)
    }
    else {
      more.push(<ExpandComponent onClick={handleExpandClick} />)
      selection.splice(limit - more.length)
    }
  }

  return (
    <React.Fragment>
      {FilterComponent && <FilterComponent onChange={handleFilterChange} />}
    <SelectComponent>
      {selection.map(props => (
        <OptionComponent
          {...props}
          onChange={handleOptionChange}
        />
      ))}
      {...more}
    </SelectComponent>
    </React.Fragment>
  )
}