import { createRegExp } from '../utils/regex'

export interface ListSelectorOptions {
  dataSet: Set<string>, // The list selection from the dataSource(s)
  dataSourceKeyToLabel: (key: string) => string
  filter?: string
}

export const ListSelector = (dataSource: object | Array<string>) => {

  const keys = Array.isArray(dataSource)
    ? dataSource
    : Object.keys(dataSource)

  return ({
    dataSet,
    dataSourceKeyToLabel,
    filter
  }: ListSelectorOptions) => {
    let selection = keys.map(key => ({
      key,
      label: dataSourceKeyToLabel(key),
      selected: dataSet.has(key),
    }))
    
    // if filter string is provided: use it
    if (typeof filter === 'string' && filter.length > 0) {
      const regex = createRegExp(filter, 'i')
      selection = selection.filter(({ label }) => regex.test(label))
    }
    
    // return sorted list with checked items first
    return selection.sort((left, right) => {
      const hasLeft = dataSet.has(left.key)
      const hasRight = dataSet.has(right.key)

      if (hasLeft !== hasRight) {
        return hasLeft ? -1 : 1
      }

      if (left.label !== right.label) {
        return left.label < right.label ? -1 : 1
      }

      return 0
    })
  }
}