import { getPresetQueries, QueryRef } from '../../../../client/js/state/query';
import { runMethodTests } from './stateTestUtils';

// all test names
const VALID = 'valid';
const INVALID = 'invalid';
const NO_GLOBAL = 'no global';

describe('Query redux state wrapper', () => {
  describe('getPresetQueries', () =>
    [{
      $name: VALID,
      $in: {
        global: {
          queries: [
            { id: '2', preset: false, query: 'y =  x', name: 'none' },
            { id: '1', preset: true, query: 'x = y', name: 'test' }
          ]
        }
      },
      $out: {
        queries: ['2', '1']
      }
    }, {
      $name: INVALID,
      $in: {},
      $out: {
        queries: []
      }
    }].forEach(testCase => it(`Should return correct preset queries for ${testCase.$name}`, () =>
      expect(getPresetQueries(testCase.$in.global).map(query => query.serialize()))
        .toEqual(testCase.$out.queries)
    ))
  );
  describe('Instance methods', () =>
    [{
      $name: VALID,
      $in: {
        id: '1',
        global: {
          queries: [
            { id: '2', preset: false, query: 'y =  x', name: 'none' },
            { id: '1', preset: true, query: 'x = y', name: 'test' }
          ]
        }
      },
      $out: {
        getId: '1',
        isPreset: true,
        getQuery: 'x = y',
        getName: 'test',
        serialize: '1'
      }
    }, {
      $name: INVALID,
      $in: {
        id: '1',
        global: {}
      },
      $out: {
        getId: '1',
        _getQuery: {}
      }
    }, {
      $name: NO_GLOBAL,
      $in: {
        id: '1'
      },
      $out: {
        getId: '1',
        _getQuery: {}
      }
    }].forEach(testCase => {
      const { id, global } = testCase.$in;
      runMethodTests(testCase, new QueryRef(id, global));
    })
  );
});
