import * as React from 'react'
import actions from './state/actions'
import reducer from './state/reducer'
import { Search, State } from './state/types'
import computeQuery from './computeQuery'

interface History {
  listen: any
  push: any
}

const initialState: State = {
}

const initializer = (state: State): State => {
  return state
}

// https://reactjs.org/docs/hooks-reference.html
const useSearch = (search: Search, history: History) => {
  const [state, dispatch] = React.useReducer(reducer, initialState, initializer)
  const memoizedQuery = React.useMemo(() => computeQuery, [search])
  
  React.useDebugValue(state, /*formatter*/state => JSON.stringify(state.search))

  history.push('/search/new');
  React.useEffect(() => {
    const unsubscribe = history.listen(() => {
      const { pathname } = window.location;
      if (!pathname.includes('/new')) {
        const id = pathname.split('/').pop();
        state.search.id = id;

        if (state.search.callback) {
          const parts = state.search.callback.split('/');
          parts[parts.length - 1] = id;
          state.search.callback = parts.join('/');
        }

        const {
          execute,
          saveOrUpdate,
          setActive
        } = actions(dispatch)      

        saveOrUpdate()
        execute()
        setActive()
        saveOrUpdate()

        // can unsubscribe here since?
        unsubscribe();
      }
    })

    // un mounting
    return unsubscribe
  })

  return state
}

export default useSearch