import { SearchFieldset, createSearchFieldset } from '../../../../client/js/state/searchFieldset';
import { FieldsetRef } from '../../../../client/js/state/fieldset';
import { FieldRef } from '../../../../client/js/state/field';

// all test names
const FIELDSET_WITH_FIELDS = 'fieldset with fields';
const FIELDSET_NO_FIELDS = 'fieldset no fields';

const runTest = (testCase, action, method) =>
  it(`Should return correct result from ${method} for ${testCase.$name}`, () =>
    expect(action(new SearchFieldset(testCase.$in))).toEqual(testCase.$out[method])
  );

it('searchFieldset.js: createFieldset()', () => {
  const searchFieldset = {
    fieldsetId: 'x',
    fieldIds: ['a', 'b'],
    visible: true,
    editionAction: null,
    lastUsedFieldsetId: 'y'
  };
  const { fieldsetId, fieldIds, visible, editionAction, lastUsedFieldsetId } = searchFieldset;
  expect(createSearchFieldset(new FieldsetRef(fieldsetId),
    fieldIds.map(id => new FieldRef(id)),
    visible,
    editionAction,
    lastUsedFieldsetId).serialize())
    .toEqual(new SearchFieldset(searchFieldset));
});

[{
  $name: FIELDSET_WITH_FIELDS,
  $in: {
    fieldsetId: 'x',
    fieldIds: ['a', 'b'],
    lastUsedFieldsetId: 'y',
    visible: true,
    setFieldset: 'z',
    setFields: ['a', 'c'],
    global: {
      fieldsetMap: {
        z: { id: 'z', fieldIds: ['c'] }
      }
    }
  },
  $out: {
    getFieldset: 'x',
    getFields: ['a', 'b'],
    setFields: {
      fieldsetId: 'x',
      fieldIds: ['a', 'c'],
      lastUsedFieldsetId: 'y',
      visible: true
    },
    setFieldset: {
      fieldsetId: 'z',
      fieldIds: ['c'],
      lastUsedFieldsetId: 'x',
      editionAction: undefined,
      visible: true
    },
    getLastUsedFieldset: 'y',
    isVisible: true,
    isCustom: false
  }
}, {
  $name: FIELDSET_NO_FIELDS,
  $in: {
    fieldsetId: 'x',
    fieldIds: [],
    setFieldset: 'x',
    setFields: ['a', 'b'],
    global: {
      fieldsetMap: {
        x: { id: 'x', fieldIds: ['a'] }
      }
    }
  },
  $out: {
    getFieldset: 'x',
    getFields: [],
    setFieldset: {
      fieldsetId: 'x',
      fieldIds: ['a'],
      lastUsedFieldsetId: undefined,
      editionAction: undefined,
      visible: undefined
    },
    getLastUsedFieldset: undefined,
    isVisible: false,
    setFields: {
      fieldsetId: 'x',
      fieldIds: ['a', 'b'],
      lastUsedFieldsetId: undefined,
      editionAction: undefined,
      visible: undefined
    },
    isCustom: false
  }
}].forEach(testCase => {
  runTest(testCase, (fieldset) => !!fieldset.isVisible(), 'isVisible');
  runTest(testCase, (fieldset) => !!fieldset.isCustom(), 'isCustom');
  ['getFieldset', 'getLastUsedFieldset'].forEach((method) =>
    runTest(testCase, (fieldset) => fieldset[method]().serialize(), method)
  );
  const getFields = (fieldset) => fieldset.getFields().map(fieldRef => fieldRef.serialize());
  runTest(testCase, getFields, 'getFields');
  const fieldsetRef = new FieldsetRef(testCase.$in.setFieldset, testCase.$in.global);
  runTest(testCase, (fieldset) => fieldset.setFieldset(fieldsetRef).serialize(), 'setFieldset');
  const fieldRefs = testCase.$in.setFields.map(id => new FieldRef(id));
  runTest(testCase, (fieldset) => fieldset.setFields(fieldRefs), 'setFields');
});
