import { DatePicker } from '../../../../client/js/state/datePicker';
import { Range } from '../../../../client/js/state/range';
import { TimelineData } from '../../../../client/js/state/timelineData';
import { Timeline, createTimeline } from '../../../../client/js/state/timeline';
import { runMethodTests } from './stateTestUtils';

// all test names
const WITH_RANGE = 'with range';
const NO_RANGE = 'no range';
const WITH_DATA = 'with data';
const UNKNOWN_TIMELINE_GROUPING = 'unknown timeline format';

describe('Timeline redux state wrapper', () =>
  [{
    $name: WITH_RANGE,
    $in: {
      collapsed: false,
      range: {
        start: 0,
        end: 0
      },
      setCollapsed: [true]
    },
    $out: {
      isCollapsed: false,
      getRange: {
        start: 0,
        end: 0
      },
      hasRange: true,
      setRange: {
        collapsed: false,
        range: undefined
      },
      setCollapsed: {
        collapsed: true,
        range: {
          start: 0,
          end: 0
        }
      },
      hasData: false,
      getData: undefined,
      shrink: {
        collapsed: false,
        range: {
          start: 0,
          end: 0
        }
      }
    }
  }, {
    $name: NO_RANGE,
    $in: {
      collapsed: true,
      setRange: [new Range({
        start: 0,
        end: 1
      })],
      setData: [new TimelineData({ events: [] })]
    },
    $out: {
      isCollapsed: true,
      hasRange: false,
      getRange: undefined,
      setRange: {
        collapsed: true,
        range: {
          start: 0,
          end: 1,
          startDate: undefined,
          endDate: undefined
        }
      },
      setData: {
        collapsed: true,
        data: { events: [] }
      }
    }
  }, {
    $name: WITH_DATA,
    $in: {
      collapsed: true,
      data: { events: [] },
      groupBy: 'seconds',
      granularity: 1,
      setData: [],
      setGranularity: [2],
      setGroupBy: ['minutes'],
      adjustTo: [new DatePicker({
        startDate: 0,
        endDate: 3600000,
        datePickerLabel: 'Custom Range'
      })]
    },
    $out: {
      getGranularity: 1,
      getGroupBy: 'seconds',
      setGranularity: {
        collapsed: true,
        data: { events: [] },
        groupBy: 'seconds',
        granularity: 2
      },
      setGroupBy: {
        collapsed: true,
        data: { events: [] },
        groupBy: 'minutes',
        granularity: 1
      },
      setData: {
        collapsed: true,
        groupBy: 'seconds',
        granularity: 1
      },
      adjustTo: {
        collapsed: true,
        data: { events: [] },
        groupBy: 'seconds',
        granularity: 30
      }
    }
  }, {
    $name: UNKNOWN_TIMELINE_GROUPING,
    $in: {
      collapsed: false,
      range: {
        start: 0,
        end: 0
      },
      groupBy: -1
    },
    $out: {
      getDateFormat: 'MMM DD YYYY, hh:mm:ssa'
    }
  }].forEach(testCase => {
    const { range, collapsed, data, granularity, groupBy } = testCase.$in;
    const timeline = new Timeline(testCase.$in);
    it(`Should create timeline equal to its serialized form for ${testCase.$name}`, () => {
      const created = createTimeline(
        range ? new Range(range) : undefined,
        collapsed,
        granularity,
        groupBy,
        data ? new TimelineData(data) : undefined
      ).serialize();
      expect(created).toEqual(timeline.serialize());
    });

    runMethodTests(testCase, timeline);
  })
);
