import { runMethodTests } from './stateTestUtils';
import { Timeline } from '../../../../client/js/state/timeline';
import { DatePicker } from '../../../../client/js/state/datePicker';
import { TimelineData, generateTimelineData } from '../../../../client/js/state/timelineData';

// all test names
const EMPTY = 'empty';
const NO_EVENTS = 'no events';
const TOTAL_COUNT = 'total count';
const SELECTED_COUNT = 'selected count';
const NON_EMPTY = 'non empty';

describe('Timeline data redux state wrapper', () =>
  [{
    $name: EMPTY,
    $in: {
      events: []
    },
    $out: {
      isEmpty: true
    }
  }, {
    $name: NO_EVENTS,
    $in: {
      events: null
    },
    $out: {
      isEmpty: true
    }
  }, {
    $name: TOTAL_COUNT,
    $in: {
      events: [
        { epoch: 1, count: 10 },
        { epoch: 2, count: 20 },
        { epoch: 3, count: 30 }
      ]
    },
    $out: {
      getTimes: [1, 2, 3],
      getCounts: [10, 20, 30],
      getTotalCount: 60,
      getMaxCount: 30,
      isEmpty: false
    }
  }, {
    $name: SELECTED_COUNT,
    $in: {
      events: [
        { epoch: 3, count: 10 },
        { epoch: 4, count: 20 },
        { epoch: 5, count: 30 }
      ],
      getTotalCount: [1, 2]
    },
    $out: {
      getTotalCount: 50,
      getTimes: [3, 4, 5],
      isEmpty: false
    }
  }].forEach((testCase) => {
    const timelineData = new TimelineData(testCase.$in);

    it(`Should correctly serialize timeline data for ${testCase.$name}`, () =>
      expect(timelineData.serialize())
        .toEqual(new TimelineData(timelineData.serialize()).serialize())
    );

    runMethodTests(testCase, timelineData);
  })
);

describe('generateTimelineData', () =>
  [{
    $name: NON_EMPTY,
    $in: {
      response: {
        columns: ['timestamp', 'count'],
        rows: [[0, 10], [2000, 20], [4000, 30]]
      },
      timeline: { groupBy: 'seconds', granularity: 1 },
      datePicker: { startDate: 0, endDate: 5000, datePickerLabel: 'Custom Range' }
    },
    $out: {
      events: [
        { epoch: 0, count: 10 },
        { epoch: 1000, count: 0 },
        { epoch: 2000, count: 20 },
        { epoch: 3000, count: 0 },
        { epoch: 4000, count: 30 },
        { epoch: 5000, count: 0 }, // TODO: should it be present?
        { epoch: 5999, count: 0 } // TODO: should it be present?
      ]
    }
  }].forEach((testCase) => it(`${testCase.$name}`, () => {
    const { response, timeline, datePicker } = testCase.$in;
    const timelineData = generateTimelineData(
      response,
      new Timeline(timeline),
      new DatePicker(datePicker)
    );
    expect(timelineData.serialize()).toEqual(testCase.$out);
  }))
);
