/**
 * Creates a regular expression using the provided pattern.
 * If the pattern is not valid for a regular expression, the pattern is escaped.
 * @param {string} pattern Pattern to use the create the regular expression.
 * @param {string} flags Flags to use on the regular expression, the default flags are "gi"
 * @returns Returns a new regular expression based on the pattern.
 */
export const createRegExp = (pattern: string, flags: string = 'gi') => {
  let regex: RegExp
  try {
    regex = new RegExp(pattern, flags)
  } catch (e) {
    // failed to create the reg exp, in this case we escape the special characters.
    const escaped = pattern.replace(/[^\w\s]/g, '\\$&')
    regex = new RegExp(escaped, flags)
  }
  return regex;
}